@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2022 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
+** Copyright (C) 2022-2023 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -26,7 +26,6 @@
 #include <array>
 #include <string>
 #include <regex>
-#include <utils/mu-util.h>
 #include <utils/mu-utils.hh>
 #include <utils/mu-error.hh>
 #include <utils/mu-option.hh>
@@ -74,11 +73,13 @@ struct Message::Private {
 static void fill_document(Message::Private& priv);
 
 static Result<struct stat>
-get_statbuf(const std::string& path)
+get_statbuf(const std::string& path, Message::Options opts = Message::Options::None)
 {
-	if (!g_path_is_absolute(path.c_str()))
+	if (none_of(opts & Message::Options::AllowRelativePath) &&
+	    !g_path_is_absolute(path.c_str()))
 		return Err(Error::Code::File, "path '%s' is not absolute",
 			   path.c_str());
+
 	if (::access(path.c_str(), R_OK) != 0)
 		return Err(Error::Code::File, "file @ '%s' is not readable",
 			   path.c_str());
@@ -98,7 +99,7 @@ get_statbuf(const std::string& path)
 Message::Message(const std::string& path,  Message::Options opts):
 	priv_{std::make_unique<Private>(opts)}
 {
-	const auto statbuf{get_statbuf(path)};
+	const auto statbuf{get_statbuf(path, opts)};
 	if (!statbuf)
 		throw statbuf.error();
 
@@ -182,23 +183,16 @@ Message::docid() const
 }
 
 
-const Mu::Sexp::List&
-Message::to_sexp_list() const
+const Mu::Sexp&
+Message::sexp() const
 {
-	return priv_->doc.sexp_list();
-}
-
-void
-Message::update_cached_sexp()
-{
-	priv_->doc.update_cached_sexp();
+	return priv_->doc.sexp();
 }
 
 Result<void>
 Message::set_maildir(const std::string& maildir)
 {
 	/* sanity check a little bit */
-
 	if (maildir.empty() ||
 	    maildir.at(0) != '/' ||
 	    (maildir.size() > 1 && maildir.at(maildir.length()-1) == '/'))
@@ -296,14 +290,14 @@ extract_tags(const MimeMessage& mime_msg)
 	constexpr std::array<std::pair<const char*, char>, 3> tag_headers = {{
 		{"X-Label", ' '}, {"X-Keywords", ','}, {"Keywords", ' '}
 	}};
-	static const auto strip_rx{std::regex("^\\s+| +$|( )\\s+")};
 
 	std::vector<std::string> tags;
 	seq_for_each(tag_headers, [&](auto&& item) {
 		if (auto&& hdr = mime_msg.header(item.first); hdr) {
 			for (auto&& tagval : split(*hdr, item.second)) {
-				tags.emplace_back(
-					std::regex_replace(tagval, strip_rx, "$1"));
+				tagval.erase(0, tagval.find_first_not_of(' '));
+				tagval.erase(tagval.find_last_not_of(' ')+1);
+				tags.emplace_back(std::move(tagval));
 			}
 		}
 	});
@@ -638,10 +632,10 @@ fill_document(Message::Private& priv)
 
 	const auto path{doc.string_value(Field::Id::Path)};
 	const auto refs{mime_msg.references()};
-        const auto& raw_message_id = mime_msg.message_id();
-        const auto message_id = raw_message_id.has_value() && !raw_message_id->empty()
-            ? *raw_message_id
-            : fake_message_id(path);
+	const auto& raw_message_id = mime_msg.message_id();
+	const auto message_id = raw_message_id.has_value() && !raw_message_id->empty()
+	    ? *raw_message_id
+	    : fake_message_id(path);
 
 	process_message(mime_msg, path, priv);
 
@@ -706,7 +700,7 @@ fill_document(Message::Private& priv)
 		case Field::Id::Size: /* already */
 			break;
 		case Field::Id::Subject:
-			doc.add(field.id, mime_msg.subject());
+			doc.add(field.id, mime_msg.subject().map(remove_ctrl));
 			break;
 		case Field::Id::Tags:
 			if (auto&& tags{extract_tags(mime_msg)}; !tags.empty())
