@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2022 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
+** Copyright (C) 2023 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -235,7 +235,7 @@ World!
 	{
 		auto&& part{message->parts().at(2)};
 		assert_equal(part.raw_filename().value(), "/tmp/file-02.bin");
-		assert_equal(part.cooked_filename().value(), "tmp-file-02.bin");
+		assert_equal(part.cooked_filename().value(), "file-02.bin");
 		assert_equal(part.mime_type().value(), "audio/ogg");
 		// file consistso of 4 bytes 4..7
 		assert_equal(part.to_string().value(), "\004\005\006\007");
@@ -249,6 +249,10 @@ World!
 	{
 		auto&& part{message->parts().at(3)};
 		g_assert_true(part.mime_type() == "message/rfc822");
+
+		const auto fname{*cache_path + "/msgpart"};
+		g_assert_cmpuint(part.to_file(fname, true).value_or(123), ==, 139);
+		g_assert_true(::access(fname.c_str(), F_OK) == 0);
 	}
 
 	{
@@ -481,8 +485,7 @@ Content-Type: message/rfc822
 )";
 	auto message{Message::make_from_text(msgtext)};
 	g_assert_true(!!message);
-
-	g_assert_true(message->cached_sexp().empty());
+	//g_assert_true(message->sexp().empty());
 }
 
 
@@ -495,6 +498,7 @@ Date: Tue, 3 May 2022 10:26:26 +0300
 Message-ID: <SADKLAJCLKDJLAS-xheQjE__+hS-3tff=pTYpMUyGiJwNGF_DA@mail.gmail.com>
 Subject: =?Windows-1252?Q?Purkuty=F6urakka?=
 To: Hello <moika@example.com>
+Cc: =?iso-8859-1?q?M=FCller=2C?= Mickey <Mickey.Mueller@example.com>
 Content-Type: multipart/mixed; boundary="000000000000e687ed05de166d71"
 
 --000000000000e687ed05de166d71
@@ -542,6 +546,12 @@ cmVmCjM1NjE4CiUlRU9GCg==
 
 	g_assert_true(message->path().empty());
 
+	/* https://groups.google.com/g/mu-discuss/c/kCtrlxMXBjo */
+	g_assert_cmpuint(message->cc().size(),==, 1);
+	assert_equal(message->cc().at(0).email, "Mickey.Mueller@example.com");
+	assert_equal(message->cc().at(0).name, "Müller, Mickey");
+	assert_equal(message->cc().at(0).display_name(), "\"Müller, Mickey\" <Mickey.Mueller@example.com>");
+
 	g_assert_true(message->bcc().empty());
 	assert_equal(message->subject(), "Purkutyöurakka");
 	assert_equal(message->body_html().value_or(""), "abc\n");
@@ -558,7 +568,6 @@ Moi,
 
 --
 )");
-	g_assert_true(message->cc().empty());
 	g_assert_cmpuint(message->date(), ==, 1651562786);
 	g_assert_true(message->flags() == (Flags::HasAttachment));
 
@@ -805,6 +814,7 @@ RU5EOlZFVkVOVA0KRU5EOlZDQUxFTkRBUg0K
 		     "Invitation: HELLO, @ Thu 9 Jan 2014 08:30 - 09:30 (william@example.com)");
 	g_assert_true(message->flags() == (Flags::Passed|Flags::Seen|
 					   Flags::HasAttachment|Flags::Calendar));
+	g_assert_cmpuint(message->body_html().value_or("").find("DETAILS"), ==, 2271);
 }
 
 
@@ -994,7 +1004,6 @@ Message-ID:
 
 abc
 )";
-
 	const auto m1{Message::make_from_text(msg1, "/foo/cur/m123:2,S")};
 	assert_valid_result(m1);
 
@@ -1007,8 +1016,11 @@ abc
 
 	/* both with absent and empty message-id, generate "random" fake one,
 	 * which must end in @mu.id */
-	g_assert_true(g_str_has_suffix(m2->message_id().c_str(), "@mu.id"));
-	g_assert_true(g_str_has_suffix(m3->message_id().c_str(), "@mu.id"));
+	const auto id2{m2->message_id()};
+	const auto id3{m3->message_id()};
+
+	g_assert_true(g_str_has_suffix(id2.c_str(), "@mu.id"));
+	g_assert_true(g_str_has_suffix(id3.c_str(), "@mu.id"));
 }
 
 
@@ -1034,6 +1046,28 @@ test_message_sanitize_maildir()
 	assert_equal(Message::sanitize_maildir("/foo/bar/cuux/"), "/foo/bar/cuux");
 }
 
+static void
+test_message_subject_with_newline()
+{
+constexpr const auto txt =
+R"(To: foo@example.com
+Subject: =?utf-8?q?Le_poids_=C3=A9conomique_de_la_chasse_:_=0A=0Ala_dette_cach?= =?utf-8?q?=C3=A9e_de_la_chasse_!?=
+Date: Mon, 24 Apr 2023 07:32:43 +0000
+
+Hello!
+)";
+	g_test_bug("2477");
+
+	const auto msg{Message::make_from_text(txt, "/foo/cur/m123:2,S")};
+	assert_valid_result(msg);
+
+	assert_equal(msg->subject(), // newlines are filtered-out
+		     "Le poids économique de la chasse : la dette cachée de la chasse !");
+	assert_equal(msg->header("Subject").value_or(""),
+		     "Le poids économique de la chasse : \n\nla dette cachée de la chasse !");
+}
+
+
 int
 main(int argc, char* argv[])
 {
@@ -1059,6 +1093,8 @@ main(int argc, char* argv[])
 			test_message_outlook_body);
 	g_test_add_func("/message/message/message-id",
 			test_message_message_id);
+	g_test_add_func("/message/message/subject-with-newline",
+			test_message_subject_with_newline);
 	g_test_add_func("/message/message/fail",
 			test_message_fail);
 	g_test_add_func("/message/message/sanitize-maildir",
