@@ -34,7 +34,7 @@
 #include "glibconfig.h"
 #include "mu-maildir.hh"
 #include "utils/mu-utils.hh"
-#include "utils/mu-util.h"
+#include "utils/mu-utils-file.hh"
 
 using namespace Mu;
 
@@ -61,7 +61,8 @@ get_dtype(struct dirent* dentry, const std::string& path, bool use_lstat)
 
 slowpath:
 #endif /*HAVE_STRUCT_DIRENT_D_TYPE*/
-	return mu_util_get_dtype(path.c_str(), use_lstat);
+
+	return determine_dtype(path, use_lstat);
 }
 
 static Mu::Result<void>
@@ -73,11 +74,11 @@ create_maildir(const std::string& path, mode_t mode)
 	std::array<std::string,3> subdirs = {"new", "cur", "tmp"};
 	for (auto&& subdir: subdirs) {
 
-		const auto fullpath{path + G_DIR_SEPARATOR_S + subdir};
+		const auto fullpath{join_paths(path, subdir)};
 
 		/* if subdir already exists, don't try to re-create
 		 * it */
-		if (mu_util_check_dir(fullpath.c_str(), TRUE, TRUE))
+		if (check_dir(fullpath, true/*readable*/, true/*writable*/))
 			continue;
 
 		int rv{g_mkdir_with_parents(fullpath.c_str(), static_cast<int>(mode))};
@@ -85,7 +86,7 @@ create_maildir(const std::string& path, mode_t mode)
 		/* note, g_mkdir_with_parents won't detect an error if
 		 * there's already such a dir, but with the wrong
 		 * permissions; so we need to check */
-		if (rv != 0 || !mu_util_check_dir(fullpath.c_str(), TRUE, TRUE))
+		if (rv != 0 || !check_dir(fullpath, true/*readable*/, true/*writable*/))
 			return Err(Error{Error::Code::File,
 					"creating dir failed for %s: %s",
 					fullpath.c_str(), g_strerror(errno)});
@@ -97,7 +98,7 @@ create_maildir(const std::string& path, mode_t mode)
 static Mu::Result<void>		/* create a noindex file if requested */
 create_noindex(const std::string& path)
 {
-	const auto noindexpath{path + G_DIR_SEPARATOR_S MU_MAILDIR_NOINDEX_FILE};
+	const auto noindexpath{join_paths(path, MU_MAILDIR_NOINDEX_FILE)};
 
 	/* note, if the 'close' failed, creation may still have succeeded...*/
 	int fd = ::creat(noindexpath.c_str(), 0644);
@@ -154,7 +155,7 @@ get_target_fullpath(const std::string& src, const std::string& targetpath,
 	if (auto&& res = check_subdir(src, in_cur); !res)
 		return Err(std::move(res.error()));
 
-	char *srcfile{g_path_get_basename(src.c_str())};
+	const auto srcfile{to_string_gchar(g_path_get_basename(src.c_str()))};
 
 	/* create targetpath; note: make the filename *cough* unique by
 	 * including a hash of the srcname in the targetname. This helps if
@@ -162,20 +163,15 @@ get_target_fullpath(const std::string& src, const std::string& targetpath,
 	 */
 	std::string fulltargetpath;
 	if (unique_names)
-		fulltargetpath = format("%s%c%s%c%u_%s",
-				    targetpath.c_str(),
-				    G_DIR_SEPARATOR, in_cur ? "cur" : "new",
-				    G_DIR_SEPARATOR,
-				    g_str_hash(src.c_str()),
-				    srcfile);
+		fulltargetpath = join_paths(targetpath,
+					    in_cur ? "cur" : "new",
+					    format("%08x-%s",
+						   g_str_hash(src.c_str()),
+						   srcfile.c_str()));
 	else
-		fulltargetpath = format("%s%c%s%c%s",
-				    targetpath.c_str(),
-				    G_DIR_SEPARATOR, in_cur ? "cur" : "new",
-				    G_DIR_SEPARATOR,
-				    srcfile);
-	g_free(srcfile);
-
+		fulltargetpath = join_paths(targetpath,
+					    in_cur ? "cur" : "new",
+					    srcfile.c_str());
 	return fulltargetpath;
 }
 
@@ -212,8 +208,7 @@ clear_links(const std::string& path, DIR* dir)
 		if (dentry->d_name[0] == '.')
 			continue; /* ignore .,.. other dotdirs */
 
-		const auto fullpath{
-			format("%s" G_DIR_SEPARATOR_S "%s",path.c_str(), dentry->d_name)};
+		const auto fullpath{join_paths(path, dentry->d_name)};
 		const auto d_type   = get_dtype(dentry, fullpath.c_str(), true/*lstat*/);
 		switch(d_type) {
 		case DT_LNK:
@@ -440,8 +435,8 @@ Mu::maildir_determine_target(const std::string&	old_path,
 
 	/* if target_mdir is empty, the src_dir does not change (though cur/
 	 * maybe become new or vice-versa) */
-	const auto dst_mdir{target_maildir.empty() ? src_mdir :
-		root_maildir_path + target_maildir};
+	const auto dst_mdir = target_maildir.empty() ? src_mdir :
+		join_paths(root_maildir_path,  target_maildir);
 
 	/* now calculate the message name (incl. its immediate parent dir) */
 	const auto dst_file{determine_dst_filename(src_file, newflags, new_name)};
@@ -454,5 +449,5 @@ Mu::maildir_determine_target(const std::string&	old_path,
 			return "new";
 	});
 
-	return dst_mdir + G_DIR_SEPARATOR_S + subdir + G_DIR_SEPARATOR_S + dst_file;
+	return join_paths(dst_mdir, subdir,dst_file);
 }
