@@ -561,23 +561,29 @@ Boo!
 	assert_equal(qr->begin().message()->path(), old_path);
 	g_assert_true(::access(old_path.c_str(), F_OK) == 0);
 
+
 	/*
 	 * mark as read, i.e. move to cur/; ensure it really moved.
 	 */
-	auto moved_msg = store.move_message(old_docid, Nothing, Flags::Seen, rename);
-	assert_valid_result(moved_msg);
-	const auto new_path = moved_msg->path();
+	auto move_opts{rename ? Store::MoveOptions::ChangeName : Store::MoveOptions::None};
+	auto moved_msgs = store.move_message(old_docid, Nothing, Flags::Seen, move_opts);
+	assert_valid_result(moved_msgs);
+	g_assert_true(moved_msgs->size() == 1);
+	const auto& moved_msg{moved_msgs->at(0).second};
+	const auto new_path = moved_msg.path();
 	if (!rename)
 		assert_equal(new_path, store.properties().root_maildir + "/inbox/cur/msg:2,S");
 	g_assert_cmpuint(store.size(), ==, 1);
 	g_assert_false(::access(old_path.c_str(), F_OK) == 0);
 	g_assert_true(::access(new_path.c_str(), F_OK) == 0);
 
-	/* also ensure thath the cached sexp for the message has been updated;
+	/* also ensure that the cached sexp for the message has been updated;
 	 * that's what mu4e uses */
-	const auto moved_sexp{moved_msg->to_sexp().to_sexp_string()};
-	/* clumsy */
-	g_assert_true(moved_sexp.find(new_path) != std::string::npos);
+	const auto moved_sexp{moved_msg.sexp()};
+	//std::cerr << "@@ " << *moved_msg << '\n';
+	g_assert_true(moved_sexp.plistp());
+	g_assert_true(moved_sexp.has_prop(":path"));
+	assert_equal(moved_sexp.get_prop(":path").string(), new_path);
 
 	/*
 	 * find new message with query, ensure it's really that new one.
@@ -622,6 +628,149 @@ test_duplicate_refresh_rename()
 	test_duplicate_refresh_real(true/*rename*/);
 }
 
+static void
+test_term_split()
+{
+	g_test_bug("2365");
+
+	// Note the fancy quote in "foo’s bar"
+	const TestMap test_msgs = {{
+			"inbox/new/msg",
+			{ R"(Message-Id: <abcde@foo.bar>
+From: "Foo Example" <bar@example.com>
+Date: Wed, 26 Oct 2022 11:01:54 -0700
+To: example@example.com
+Subject: foo’s bar
+
+Boo!
+)"},
+		}};
+
+	TempDir tdir;
+	auto store{make_test_store(tdir.path(), test_msgs, {})};
+	/* true: match; false: no match */
+	const auto cases = std::array<std::pair<const char*, bool>, 6>{{
+		{"subject:foo's", true},
+		{"subject:foo*", true},
+		{"subject:/foo/", true},
+		{"subject:/foo’s/", true}, /* <-- breaks before PR #2365 */
+		{"subject:/foo.*bar/", true},  /* <-- breaks before PR #2365 */
+		{"subject:/foo’s bar/", false}, /* <-- no matching yet */
+	}};
+
+	for (auto&& test: cases) {
+		g_debug("query: %s", test.first);
+		auto qr = store.run_query(test.first);
+		assert_valid_result(qr);
+		if (test.second)
+			g_assert_cmpuint(qr->size(), ==, 1);
+		else
+			g_assert_true(qr->empty());
+	}
+}
+
+static void
+test_related_dup_threaded()
+{
+	// test message sent to self, and copy of received msg.
+
+	const auto test_msg = R"(From: "Edward Mallory" <ed@leviathan.gb>
+To: "Laurence Oliphant <oli@hotmail.com>
+Subject: Boo
+Date: Wed, 07 Dec 2022 18:38:06 +0200
+Message-ID: <875yentbhg.fsf@djcbsoftware.nl>
+MIME-Version: 1.0
+Content-Type: text/plain
+
+Boo!
+)";
+	const TestMap test_msgs = {
+		{"sent/cur/msg1", test_msg },
+		{"inbox/cur/msg1", test_msg },
+		{"inbox/cur/msg2", test_msg }};
+
+	TempDir tdir;
+	auto store{make_test_store(tdir.path(), test_msgs, {})};
+
+	g_assert_cmpuint(store.size(), ==, 3);
+
+
+	// normal query should give 2
+	{
+		auto qr = store.run_query("maildir:/inbox", Field::Id::Date,
+					  QueryFlags::None);
+		assert_valid_result(qr);
+		g_assert_cmpuint(qr->size(), ==, 2);
+	}
+
+	// a related query should give 3
+	{
+		auto qr = store.run_query("maildir:/inbox", Field::Id::Date,
+					  QueryFlags::IncludeRelated);
+		assert_valid_result(qr);
+		g_assert_cmpuint(qr->size(), ==, 3);
+	}
+
+	// a related/threading query should give 3.
+	{
+		auto qr = store.run_query("maildir:/inbox", Field::Id::Date,
+					  QueryFlags::IncludeRelated | QueryFlags::Threading);
+		assert_valid_result(qr);
+		g_assert_cmpuint(qr->size(), ==, 3);
+	}
+}
+
+
+static void
+test_html()
+{
+	// test message sent to self, and copy of received msg.
+
+	const auto test_msg = R"(From: Test <test@example.com>
+To: abc@example.com
+Date: Mon, 23 May 2011 10:53:45 +0200
+Subject: vla
+MIME-Version: 1.0
+Content-Type: multipart/alternative;
+	boundary="_=aspNetEmail=_5ed4592191214c7a99bd7f6a3a0f077d"
+Message-ID: <10374608.109906.11909.20115aabbccdd.MSGID@mailinglijst.nl>
+
+--_=aspNetEmail=_5ed4592191214c7a99bd7f6a3a0f077d
+Content-Type: text/plain; charset="iso-8859-15"
+Content-Transfer-Encoding: quoted-printable
+
+text
+
+--_=aspNetEmail=_5ed4592191214c7a99bd7f6a3a0f077d
+Content-Type: text/html; charset="iso-8859-15"
+Content-Transfer-Encoding: quoted-printable
+
+html
+
+--_=aspNetEmail=_5ed4592191214c7a99bd7f6a3a0f077d--
+)";
+	const TestMap test_msgs = {{"inbox/cur/msg1", test_msg }};
+
+	TempDir tdir;
+	auto store{make_test_store(tdir.path(), test_msgs, {})};
+	g_assert_cmpuint(store.size(), ==, 1);
+
+	{
+		auto qr = store.run_query("body:text", Field::Id::Date,
+					  QueryFlags::None);
+		assert_valid_result(qr);
+		g_assert_cmpuint(qr->size(), ==, 1);
+	}
+
+	{
+		auto qr = store.run_query("body:html", Field::Id::Date,
+					  QueryFlags::None);
+		assert_valid_result(qr);
+		g_assert_cmpuint(qr->size(), ==, 1);
+	}
+
+}
+
 
 int
 main(int argc, char* argv[])
@@ -643,6 +792,11 @@ main(int argc, char* argv[])
 			test_duplicate_refresh);
 	g_test_add_func("/store/query/duplicate-refresh-rename",
 			test_duplicate_refresh_rename);
+	g_test_add_func("/store/query/term-split",
+			test_term_split);
+	g_test_add_func("/store/query/related-dup-threaded",
+			test_related_dup_threaded);
+	g_test_add_func("/store/query/html", test_html);
 
 	return g_test_run();
 }
