@@ -22,6 +22,7 @@
 #include <glib.h>
 #include <stdlib.h>
 #include <thread>
+#include <array>
 #include <unistd.h>
 #include <time.h>
 #include <fstream>
@@ -32,6 +33,7 @@
 #include "mu-store.hh"
 #include "utils/mu-result.hh"
 #include <utils/mu-utils.hh>
+#include <utils/mu-utils-file.hh>
 #include "mu-maildir.hh"
 
 using namespace Mu;
@@ -53,6 +55,62 @@ test_store_ctor_dtor()
 			store->properties().schema_version.c_str());
 }
 
+
+
+static void
+test_store_reinit()
+{
+	TempDir tempdir;
+	{
+		Store::Config conf{};
+		conf.max_message_size = 1234567;
+		conf.batch_size	      = 7654321;
+
+		StringVec my_addresses{ "foo@example.com", "bar@example.com" };
+
+		auto store{Store::make_new(tempdir.path(), MuTestMaildir, my_addresses, conf)};
+		assert_valid_result(store);
+
+		g_assert_true(store->empty());
+		g_assert_cmpuint(0, ==, store->size());
+
+		g_assert_cmpstr(MU_STORE_SCHEMA_VERSION, ==,
+				store->properties().schema_version.c_str());
+
+		const auto msgpath{MuTestMaildir + "/cur/1283599333.1840_11.cthulhu!2,"};
+		const auto id = store->add_message(msgpath);
+		assert_valid_result(id);
+		g_assert_true(store->contains_message(msgpath));
+		g_assert_cmpuint(store->size(), ==, 1);
+	}
+
+	//now let's reinitialize it.
+	{
+		auto store{Store::make(tempdir.path(),
+				       Store::Options::Writable|Store::Options::ReInit)};
+
+		assert_valid_result(store);
+		g_assert_true(store->empty());
+
+		assert_equal(store->properties().database_path, tempdir.path());
+		g_assert_cmpuint(store->properties().batch_size,==,7654321);
+		g_assert_cmpuint(store->properties().max_message_size,==,1234567);
+
+		const auto addrs{store->properties().personal_addresses};
+		g_assert_cmpuint(addrs.size(),==,2);
+		g_assert_true(seq_some(addrs, [](auto&& a){return a=="foo@example.com";}));
+		g_assert_true(seq_some(addrs, [](auto&& a){return a=="bar@example.com";}));
+
+		const auto msgpath{MuTestMaildir + "/cur/1283599333.1840_11.cthulhu!2,"};
+		const auto id = store->add_message(msgpath);
+		assert_valid_result(id);
+		g_assert_true(store->contains_message(msgpath));
+		g_assert_cmpuint(store->size(), ==, 1);
+	}
+}
+
+
+
 static void
 test_store_add_count_remove()
 {
@@ -69,8 +127,6 @@ test_store_add_count_remove()
 	g_assert_cmpuint(store->size(), ==, 1);
 	g_assert_true(store->contains_message(msgpath));
 
-	g_assert_true(store->contains_message(msgpath));
-
 	const auto id2 = store->add_message(MuTestMaildir2 + "/bar/cur/mail3");
 	g_assert_false(!!id2); // wrong maildir.
 	store->commit();
@@ -314,22 +370,99 @@ Yes, that would be excellent.
 	 const auto oldpath{msg->path()};
 	 assert_equal(msg->subject(), "Re: multi-eq hash tables");
 	 g_assert_true(msg->docid() != 0);
-	 g_debug("%s", msg->to_sexp().to_sexp_string().c_str());
+	 g_debug("%s", msg->sexp().to_string().c_str());
 
 	 // Move the message from new->cur
 	 std::this_thread::sleep_for(1s); /* ctime should change */
-	 const auto msg3 = store->move_message(msg->docid(), {}, Flags::Seen);
-	 assert_valid_result(msg3);
-	 assert_equal(msg3->maildir(), "/a");
-	 assert_equal(msg3->path(), tempdir2.path() + "/Maildir/a/cur/msg:2,S");
-	 g_assert_true(::access(msg3->path().c_str(), R_OK)==0);
+	 const auto msgs3 = store->move_message(msg->docid(), {}, Flags::Seen);
+	 assert_valid_result(msgs3);
+	 g_assert_true(msgs3->size() == 1);
+	 const auto& msg3{msgs3->at(0).second};
+	 assert_equal(msg3.maildir(), "/a");
+	 assert_equal(msg3.path(), tempdir2.path() + "/Maildir/a/cur/msg:2,S");
+	 g_assert_true(::access(msg3.path().c_str(), R_OK)==0);
 	 g_assert_false(::access(oldpath.c_str(), R_OK)==0);
 
-	 g_debug("%s", msg3->to_sexp().to_sexp_string().c_str());
+	 g_debug("%s", msg3.sexp().to_string().c_str());
 	 g_assert_cmpuint(store->size(), ==, 1);
 }
 
 
+
+static void
+test_store_move_dups()
+{
+	 const std::string msg_text =
+R"(From: Valentine Michael Smith <mike@example.com>
+To: Raul Endymion <raul@example.com>
+Subject: Re: multi-eq hash tables
+Date: Tue, 03 May 2022 20:58:02 +0200
+Message-ID: <87h766tzzz.fsf@gnus.org>
+
+Yes, that would be excellent.
+)";
+	 TempDir tempdir2;
+
+	 // create a message file + dups
+	 const auto res1 = maildir_mkdir(tempdir2.path() + "/Maildir/a");
+	 assert_valid_result(res1);
+	 const auto res2 = maildir_mkdir(tempdir2.path() + "/Maildir/b");
+	 assert_valid_result(res2);
+
+	 auto msg1_path = tempdir2.path() + "/Maildir/a/new/msg123";
+	 auto msg2_path = tempdir2.path() + "/Maildir/a/cur/msgabc:2,S";
+	 auto msg3_path = tempdir2.path() + "/Maildir/b/cur/msgdef:2,RS";
+
+	 TempDir tempdir;
+	 auto store{Store::make_new(tempdir.path(), tempdir2.path() + "/Maildir", {}, {})};
+	 assert_valid_result(store);
+
+	 std::vector<Store::Id> ids;
+	 for (auto&& p: {msg1_path, msg2_path, msg3_path}) {
+			 std::ofstream output{p};
+			 output.write(msg_text.c_str(), msg_text.size());
+			 output.close();
+			 auto res = store->add_message(p);
+			 assert_valid_result(res);
+			 ids.emplace_back(*res);
+	 }
+	 g_assert_cmpuint(store->size(), ==, 3);
+
+	 // mark main message (+ dups) as seen
+	 auto mres = store->move_message(ids.at(0), {},
+					 Flags::Seen | Flags::Flagged | Flags::Passed,
+					 Store::MoveOptions::DupFlags);
+	 assert_valid_result(mres);
+	 // al three dups should have been updated
+	 g_assert_cmpuint(mres->size(), ==, 3);
+	 // first should be the  original
+	 g_assert_cmpuint(mres->at(0).first, ==, ids.at(0));
+	 { // Message 1
+		 const Message& msg = mres->at(0).second;
+		 assert_equal(msg.path(), tempdir2.path() + "/Maildir/a/cur/msg123:2,FPS");
+		 g_assert_true(msg.flags() == (Flags::Seen|Flags::Flagged|Flags::Passed));
+	 }
+	 // note: Seen and Passed should be added to msg2/3, but Flagged shouldn't
+	 // msg3 should loose its R flag.
+
+	 auto check_msg2 = [&](const Message& msg) {
+		 assert_equal(msg.path(), tempdir2.path() + "/Maildir/a/cur/msgabc:2,PS");
+	 };
+
+	 auto check_msg3 = [&](const Message& msg) {
+		 assert_equal(msg.path(), tempdir2.path() + "/Maildir/b/cur/msgdef:2,PS");
+	 };
+
+	 if (mres->at(1).first == ids.at(1)) {
+		 check_msg2(mres->at(1).second);
+		 check_msg3(mres->at(2).second);
+	 } else  {
+		 check_msg2(mres->at(2).second);
+		 check_msg3(mres->at(1).second);
+	 }
+}
+
+
 static void
 test_store_fail()
 {
@@ -346,19 +479,20 @@ test_store_fail()
 	}
 }
 
-
 int
 main(int argc, char* argv[])
 {
 	mu_test_init(&argc, &argv);
 
 	g_test_add_func("/store/ctor-dtor", test_store_ctor_dtor);
+	g_test_add_func("/store/reinit", test_store_reinit);
 	g_test_add_func("/store/add-count-remove", test_store_add_count_remove);
 	g_test_add_func("/store/message/mailing-list",
 			test_message_mailing_list);
 	g_test_add_func("/store/message/attachments",
 			test_message_attachments);
-	g_test_add_func("/store/index/move", test_index_move);
+	g_test_add_func("/store/index/index-move", test_index_move);
+	g_test_add_func("/store/index/move-dups", test_store_move_dups);
 	g_test_add_func("/store/index/fail", test_store_fail);
 
 	return g_test_run();
