@@ -26,12 +26,11 @@
 
 #include <unistd.h>
 
-#include "mu-runtime.hh"
 #include "mu-cmd.hh"
 #include "mu-server.hh"
 
 #include "utils/mu-utils.hh"
-#include "utils/mu-command-parser.hh"
+#include "utils/mu-command-handler.hh"
 #include "utils/mu-readline.hh"
 
 using namespace Mu;
@@ -82,15 +81,15 @@ cookie(size_t n)
 }
 
 static void
-output_sexp_stdout(Sexp&& sexp, Server::OutputFlags flags)
+output_sexp_stdout(const Sexp& sexp, Server::OutputFlags flags)
 {
 	/* if requested, insert \n between list elements; note:
 	 * is _not_ inherited by children */
+	Sexp::Format fopts{};
 	if (any_of(flags & Server::OutputFlags::SplitList))
-		sexp.formatting_opts |= Sexp::FormattingOptions::SplitList;
-
-	const auto str{sexp.to_sexp_string()};
+		fopts |= Sexp::Format::SplitList;
 
+	const auto str{sexp.to_string(fopts)};
 	cookie(str.size() + 1);
 	if (G_UNLIKELY(::puts(str.c_str()) < 0)) {
 		g_critical("failed to write output '%s'", str.c_str());
@@ -104,47 +103,42 @@ output_sexp_stdout(Sexp&& sexp, Server::OutputFlags flags)
 static void
 report_error(const Mu::Error& err) noexcept
 {
-	Sexp::List e;
-
-	e.add_prop(":error", Sexp::make_number(static_cast<size_t>(err.code())));
-	e.add_prop(":message", Sexp::make_string(err.what()));
-
-	output_sexp_stdout(Sexp::make_list(std::move(e)),
+	output_sexp_stdout(Sexp(":error"_sym, Error::error_number(err.code()),
+				":message"_sym, err.what()),
 			   Server::OutputFlags::Flush);
 }
 
 
 Result<void>
-Mu::mu_cmd_server(const MuConfig* opts) try {
+Mu::mu_cmd_server(const Mu::Options& opts) try {
 
-	auto store = Store::make(mu_runtime_path(MU_RUNTIME_PATH_XAPIANDB),
+	auto store = Store::make(opts.runtime_path(RuntimePath::XapianDb),
 				 Store::Options::Writable);
 	if (!store)
 		return Err(store.error());
 
 	Server server{*store, output_sexp_stdout};
-	g_message("created server with store @ %s; maildir @ %s; debug-mode %s"
+	g_message("created server with store @ %s; maildir @ %s; debug-mode %s;"
 		  "readline: %s",
 		  store->properties().database_path.c_str(),
 		  store->properties().root_maildir.c_str(),
-		  opts->debug ? "yes" : "no",
+		  opts.debug ? "yes" : "no",
 		  have_readline() ? "yes" : "no");
 
 	tty = ::isatty(::fileno(stdout));
-	const auto eval = std::string{opts->commands ? "(help :full t)"
-				      : opts->eval   ? opts->eval
-						     : ""};
+	const auto eval = std::string{opts.server.commands ? "(help :full t)" : opts.server.eval};
 	if (!eval.empty()) {
 		server.invoke(eval);
 		return Ok();
 	}
 
 	// Note, the readline stuff is inactive unless on a tty.
-	const auto histpath{std::string{mu_runtime_path(MU_RUNTIME_PATH_CACHE)} + "/history"};
+	const auto histpath{opts.runtime_path(RuntimePath::Cache) + "/history"};
 	setup_readline(histpath, 50);
 
 	install_sig_handler();
-	std::cout << ";; Welcome to the " << PACKAGE_STRING << " command-server\n"
+	std::cout << ";; Welcome to the " << PACKAGE_STRING << " command-server"
+		  << (opts.debug ? " (debug-mode)" : "") << '\n'
 		  << ";; Use (help) to get a list of commands, (quit) to quit.\n";
 
 	bool do_quit{};
