@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2008-2022 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
+** Copyright (C) 2008-2023 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -25,6 +25,7 @@
 #include <errno.h>
 
 #include <stdlib.h>
+#include <type_traits>
 #include <unistd.h>
 #include <string.h>
 
@@ -33,6 +34,7 @@
 #include "mu-query.hh"
 #include "utils/mu-result.hh"
 #include "utils/mu-utils.hh"
+#include "utils/mu-utils-file.hh"
 
 using namespace Mu;
 
@@ -47,8 +49,8 @@ fill_database(void)
 	GError* err;
 
 	cmdline = g_strdup_printf("/bin/sh -c '"
-				  "%s init  --muhome=%s --maildir=%s --quiet; "
-				  "%s index --muhome=%s  --quiet'",
+				  "%s --quiet init  --muhome=%s --maildir=%s ; "
+				  "%s --quiet index --muhome=%s '",
 				  MU_PROGRAM,
 				  DBPATH.c_str(),
 				  MU_TESTMAILDIR2,
@@ -91,8 +93,7 @@ search_func(const char* query, unsigned expected)
 	cmdline = g_strdup_printf("%s find --muhome=%s %s", MU_PROGRAM,
 				  DBPATH.c_str(), query);
 
-	g_message("[%u] %s", expected, query);
-
+	g_debug("%s", cmdline);
 	g_assert(g_spawn_command_line_sync(cmdline, &output, &erroutput, NULL, NULL));
 	if (g_test_verbose())
 		g_print("\nOutput:\n%s", output);
@@ -151,15 +152,29 @@ test_mu_find_01(void)
 static void
 test_mu_find_02(void)
 {
-	search("bull", 1);
+	/* when matching html as if it were text,
+	 * 'bull' is also matched in arto.eml, &bull;
+	 */
+	// search("bull", 1);
+	// search("bull m:foo", 0);
+	// search("bull m:/foo", 1);
+	// search("bull m:/Foo", 1);
+	// search("bull flag:attach", 1);
+	// search("bull flag:a", 1);
+
+	search("bull", 2);
 	search("bull m:foo", 0);
-	search("bull m:/foo", 1);
-	search("bull m:/Foo", 1);
+	search("bull m:/foo", 2);
+	search("bull m:/Foo", 2);
 	search("bull flag:attach", 1);
 	search("bull flag:a", 1);
+
+
 	search("g:x", 0);
 	search("flag:encrypted", 0);
 	search("flag:attach", 1);
+
+	search("i:3BE9E6535E0D852173@emss35m06.us.lmco.com", 1);
 }
 
 static void
@@ -192,17 +207,9 @@ test_mu_find_text_in_rfc822(void)
 }
 
 /* some more tests */
-static void
-test_mu_find_03(void)
-{
-	search("bull", 1);
-	search("bull m:foo", 0);
-	search("bull m:/foo", 1);
-	search("i:3BE9E6535E0D852173@emss35m06.us.lmco.com", 1);
-}
 
 static void /* error cases */
-test_mu_find_04(void)
+test_mu_find_03()
 {
 	gchar *cmdline, *erroutput;
 
@@ -222,59 +229,60 @@ test_mu_find_04(void)
 	g_free(cmdline);
 }
 
-G_GNUC_UNUSED static void
+static void
 test_mu_find_links(void)
 {
-	gchar *cmdline, *output, *erroutput, *tmpdir;
-
-	tmpdir = test_mu_common_get_random_tmpdir();
-
-	cmdline = g_strdup_printf("%s find --muhome=%s --format=links --linksdir=%s "
-				  "mime:message/rfc822",
-				  MU_PROGRAM,
-				  DBPATH.c_str(),
-				  tmpdir);
-
+	char *output,  *erroutput;
+	const auto tmpdir{test_random_tmpdir()};
+	auto cmdline = format("%s find --muhome=%s --format=links --linksdir=%s "
+			      "mime:message/rfc822", MU_PROGRAM, DBPATH.c_str(),  tmpdir.c_str());
 	if (g_test_verbose())
-		g_print("cmdline: %s\n", cmdline);
+		g_print("cmdline: %s\n", cmdline.c_str());
 
-	g_assert(g_spawn_command_line_sync(cmdline, &output, &erroutput, NULL, NULL));
+	g_assert(g_spawn_command_line_sync(cmdline.c_str(), &output, &erroutput, NULL, NULL));
 	/* there should be no errors */
 	g_assert_cmpuint(newlines_in_output(output), ==, 0);
 	g_assert_cmpuint(newlines_in_output(erroutput), ==, 0);
 	g_free(output);
 	g_free(erroutput);
+	output = erroutput = NULL;
+
+	/* furthermore, two symlinks should be there */
+	const auto f1{format("%s/cur/rfc822.1", tmpdir.c_str())};
+	const auto f2{format("%s/cur/rfc822.2", tmpdir.c_str())};
+
+
+	g_assert_cmpuint(determine_dtype(f1.c_str(), true), ==, DT_LNK);
+	g_assert_cmpuint(determine_dtype(f2.c_str(), true), ==, DT_LNK);
 
 	/* now we try again, we should get a line of error output,
 	 * when we find the first target file already exists */
 
 	if (g_test_verbose())
-		g_print("cmdline: %s\n", cmdline);
+		g_print("cmdline: %s\n", cmdline.c_str());
 
-	g_assert(g_spawn_command_line_sync(cmdline, &output, &erroutput, NULL, NULL));
+	g_assert(g_spawn_command_line_sync(cmdline.c_str(), &output, &erroutput, NULL, NULL));
 	g_assert_cmpuint(newlines_in_output(output), ==, 0);
 	g_assert_cmpuint(newlines_in_output(erroutput), ==, 1);
 	g_free(output);
 	g_free(erroutput);
+	output = erroutput = NULL;
 
 	/* now we try again with --clearlinks, and the we should be
 	 * back to 0 errors */
-	g_free(cmdline);
-	cmdline = g_strdup_printf("%s find --muhome=%s --format=links --linksdir=%s --clearlinks "
-				  "mime:message/rfc822",
-				  MU_PROGRAM,
-				  DBPATH.c_str(),
-				  tmpdir);
-	g_assert(g_spawn_command_line_sync(cmdline, &output, &erroutput, NULL, NULL));
+	cmdline = format("%s find --muhome=%s --format=links --linksdir=%s --clearlinks "
+				   "mime:message/rfc822", MU_PROGRAM, DBPATH.c_str(),  tmpdir.c_str());
+
+	g_assert(g_spawn_command_line_sync(cmdline.c_str(), &output, &erroutput, NULL, NULL));
 	if (g_test_verbose())
-		g_print("cmdline: %s\n", cmdline);
+		g_print("cmdline: %s\n", cmdline.c_str());
 	g_assert_cmpuint(newlines_in_output(output), ==, 0);
 	g_assert_cmpuint(newlines_in_output(erroutput), ==, 0);
 	g_free(output);
 	g_free(erroutput);
 
-	g_free(cmdline);
-	g_free(tmpdir);
+	g_assert_cmpuint(determine_dtype(f1.c_str(), true), ==, DT_LNK);
+	g_assert_cmpuint(determine_dtype(f2.c_str(), true), ==, DT_LNK);
 }
 
 /* some more tests */
@@ -299,52 +307,51 @@ test_mu_find_maildir_special(void)
 /*	search ("subject:atoms", 1); */
 /* } */
 
-static void
-test_mu_extract_01(void)
-{
-	gchar *cmdline, *output, *erroutput, *tmpdir;
+// static void
+// test_mu_extract_01(void)
+// {
+//	gchar *cmdline, *output, *erroutput, *tmpdir;
 
-	tmpdir = test_mu_common_get_random_tmpdir();
-	g_assert(g_mkdir_with_parents(tmpdir, 0700) == 0);
+//	tmpdir = test_mu_common_get_random_tmpdir();
+//	g_assert(g_mkdir_with_parents(tmpdir, 0700) == 0);
 
-	cmdline = g_strdup_printf("%s extract --muhome=%s %s%cFoo%ccur%cmail5",
-				  MU_PROGRAM,
-				  tmpdir,
-				  MU_TESTMAILDIR2,
-				  G_DIR_SEPARATOR,
-				  G_DIR_SEPARATOR,
-				  G_DIR_SEPARATOR);
+//	cmdline = g_strdup_printf("%s extract %s%cFoo%ccur%cmail5",
+//				  MU_PROGRAM,
+//				  MU_TESTMAILDIR2,
+//				  G_DIR_SEPARATOR,
+//				  G_DIR_SEPARATOR,
+//				  G_DIR_SEPARATOR);
 
-	if (g_test_verbose())
-		g_print("cmd: %s\n", cmdline);
+//	if (g_test_verbose())
+//		g_print("cmd: %s\n", cmdline);
 
-	output = erroutput = NULL;
-	g_assert(g_spawn_command_line_sync(cmdline, &output, &erroutput, NULL, NULL));
+//	output = erroutput = NULL;
+//	g_assert(g_spawn_command_line_sync(cmdline, &output, &erroutput, NULL, NULL));
 
-	// ERROR:test-mu-cmd.cc:347:void test_mu_extract_01(): assertion failed (output ==
-	// "MIME-parts in this message:\n" " 1 <none> text/plain [<none>] (27 bytes)\n" " 2
-	// sittingbull.jpg image/jpeg [inline] (23.9\302\240kB)\n" " 3 custer.jpg image/jpeg
-	// [inline] (21.6\302\240kB)\n"):
-	// ("MIME-parts in this message:\n 1 <none> text/plain [<none>] (27 bytes)\n 2
-	// sittingbull.jpg image/jpeg [inline] (23.9 kB)\n 3 custer.jpg image/jpeg [inline] (21.6
-	// kB)\n" == "MIME-parts in this message:\n 1 <none> text/plain [<none>] (27 bytes)\n 2
-	// sittingbull.jpg image/jpeg [inline] (23.9\302\240kB)\n 3 custer.jpg image/jpeg [inline]
-	// (21.6\302\240kB)\n")
-	/* g_assert_cmpstr (output, */
-	/*		 ==, */
-	/*		 "MIME-parts in this message:\n" */
-	/*		 "  1 <none> text/plain [<none>] (27 bytes)\n" */
-	/*		 "  2 sittingbull.jpg image/jpeg [inline] (23.9\302\240kB)\n" */
-	/*		 "  3 custer.jpg image/jpeg [inline] (21.6\302\240kB)\n"); */
+//	// ERROR:test-mu-cmd.cc:347:void test_mu_extract_01(): assertion failed (output ==
+//	// "MIME-parts in this message:\n" " 1 <none> text/plain [<none>] (27 bytes)\n" " 2
+//	// sittingbull.jpg image/jpeg [inline] (23.9\302\240kB)\n" " 3 custer.jpg image/jpeg
+//	// [inline] (21.6\302\240kB)\n"):
+//	// ("MIME-parts in this message:\n 1 <none> text/plain [<none>] (27 bytes)\n 2
+//	// sittingbull.jpg image/jpeg [inline] (23.9 kB)\n 3 custer.jpg image/jpeg [inline] (21.6
+//	// kB)\n" == "MIME-parts in this message:\n 1 <none> text/plain [<none>] (27 bytes)\n 2
+//	// sittingbull.jpg image/jpeg [inline] (23.9\302\240kB)\n 3 custer.jpg image/jpeg [inline]
+//	// (21.6\302\240kB)\n")
+//	/* g_assert_cmpstr (output, */
+//	/*		 ==, */
+//	/*		 "MIME-parts in this message:\n" */
+//	/*		 "  1 <none> text/plain [<none>] (27 bytes)\n" */
+//	/*		 "  2 sittingbull.jpg image/jpeg [inline] (23.9\302\240kB)\n" */
+//	/*		 "  3 custer.jpg image/jpeg [inline] (21.6\302\240kB)\n"); */
 
-	/* we expect zero lines of error output */
-	g_assert_cmpuint(newlines_in_output(erroutput), ==, 0);
+//	/* we expect zero lines of error output */
+//	g_assert_cmpuint(newlines_in_output(erroutput), ==, 0);
 
-	g_free(output);
-	g_free(erroutput);
-	g_free(cmdline);
-	g_free(tmpdir);
-}
+//	g_free(output);
+//	g_free(erroutput);
+//	g_free(cmdline);
+//	g_free(tmpdir);
+// }
 
 static gint64
 get_file_size(const char* path)
@@ -354,7 +361,7 @@ get_file_size(const char* path)
 
 	rv = stat(path, &statbuf);
 	if (rv != 0) {
-		/* g_warning ("error: %s", g_strerror (errno)); */
+		g_debug ("error: %s", g_strerror (errno));
 		return -1;
 	}
 
@@ -372,11 +379,10 @@ test_mu_extract_02(void)
 
 	g_assert(g_mkdir_with_parents(tmpdir, 0700) == 0);
 
-	cmdline = g_strdup_printf("%s extract --muhome=%s -a "
+	cmdline = g_strdup_printf("%s extract -a "
 				  "--target-dir=%s %s%cFoo%ccur%cmail5",
 				  MU_PROGRAM,
 				  tmpdir,
-				  tmpdir,
 				  MU_TESTMAILDIR2,
 				  G_DIR_SEPARATOR,
 				  G_DIR_SEPARATOR,
@@ -399,7 +405,6 @@ test_mu_extract_02(void)
 	size = get_file_size(att2);
 	g_assert_cmpint(size, ==, 17674);
 
-
 	g_free(output);
 	g_free(tmpdir);
 	g_free(cmdline);
@@ -418,11 +423,10 @@ test_mu_extract_03(void)
 
 	g_assert(g_mkdir_with_parents(tmpdir, 0700) == 0);
 
-	cmdline = g_strdup_printf("%s extract --muhome=%s --parts 3 "
+	cmdline = g_strdup_printf("%s extract --parts 3 "
 				  "--target-dir=%s %s%cFoo%ccur%cmail5",
 				  MU_PROGRAM,
 				  tmpdir,
-				  tmpdir,
 				  MU_TESTMAILDIR2,
 				  G_DIR_SEPARATOR,
 				  G_DIR_SEPARATOR,
@@ -459,11 +463,10 @@ test_mu_extract_overwrite(void)
 
 	g_assert(g_mkdir_with_parents(tmpdir, 0700) == 0);
 
-	cmdline = g_strdup_printf("%s extract --muhome=%s -a "
+	cmdline = g_strdup_printf("%s extract -a "
 				  "--target-dir=%s %s%cFoo%ccur%cmail5",
 				  MU_PROGRAM,
 				  tmpdir,
-				  tmpdir,
 				  MU_TESTMAILDIR2,
 				  G_DIR_SEPARATOR,
 				  G_DIR_SEPARATOR,
@@ -491,11 +494,10 @@ test_mu_extract_overwrite(void)
 
 	g_free(cmdline);
 	/* this should work now, because we have specified --overwrite */
-	cmdline = g_strdup_printf("%s extract --muhome=%s -a --overwrite "
+	cmdline = g_strdup_printf("%s extract -a --overwrite "
 				  "--target-dir=%s %s%cFoo%ccur%cmail5",
 				  MU_PROGRAM,
 				  tmpdir,
-				  tmpdir,
 				  MU_TESTMAILDIR2,
 				  G_DIR_SEPARATOR,
 				  G_DIR_SEPARATOR,
@@ -522,12 +524,11 @@ test_mu_extract_by_name(void)
 
 	g_assert(g_mkdir_with_parents(tmpdir, 0700) == 0);
 
-	cmdline = g_strdup_printf("%s extract --muhome=%s "
+	cmdline = g_strdup_printf("%s extract "
 				  "--target-dir=%s %s%cFoo%ccur%cmail5 "
 				  "sittingbull.jpg",
 				  MU_PROGRAM,
 				  tmpdir,
-				  tmpdir,
 				  MU_TESTMAILDIR2,
 				  G_DIR_SEPARATOR,
 				  G_DIR_SEPARATOR,
@@ -559,9 +560,8 @@ test_mu_view_01(void)
 	tmpdir = test_mu_common_get_random_tmpdir();
 	g_assert(g_mkdir_with_parents(tmpdir, 0700) == 0);
 
-	cmdline = g_strdup_printf("%s view --muhome=%s %s%cbar%ccur%cmail4",
+	cmdline = g_strdup_printf("%s view %s%cbar%ccur%cmail4",
 				  MU_PROGRAM,
-				  tmpdir,
 				  MU_TESTMAILDIR2,
 				  G_DIR_SEPARATOR,
 				  G_DIR_SEPARATOR,
@@ -613,11 +613,10 @@ test_mu_view_multi(void)
 	tmpdir = test_mu_common_get_random_tmpdir();
 	g_assert(g_mkdir_with_parents(tmpdir, 0700) == 0);
 
-	cmdline = g_strdup_printf("%s view --muhome=%s "
+	cmdline = g_strdup_printf("%s view "
 				  "%s%cbar%ccur%cmail5 "
 				  "%s%cbar%ccur%cmail5",
 				  MU_PROGRAM,
-				  tmpdir,
 				  MU_TESTMAILDIR2,
 				  G_DIR_SEPARATOR,
 				  G_DIR_SEPARATOR,
@@ -649,11 +648,10 @@ test_mu_view_multi_separate(void)
 	tmpdir = test_mu_common_get_random_tmpdir();
 	g_assert(g_mkdir_with_parents(tmpdir, 0700) == 0);
 
-	cmdline = g_strdup_printf("%s view --terminate --muhome=%s "
+	cmdline = g_strdup_printf("%s view --terminate "
 				  "%s%cbar%ccur%cmail5 "
 				  "%s%cbar%ccur%cmail5",
 				  MU_PROGRAM,
-				  tmpdir,
 				  MU_TESTMAILDIR2,
 				  G_DIR_SEPARATOR,
 				  G_DIR_SEPARATOR,
@@ -716,10 +714,9 @@ test_mu_mkdir_01(void)
 	tmpdir = test_mu_common_get_random_tmpdir();
 	g_assert(g_mkdir_with_parents(tmpdir, 0700) == 0);
 
-	cmdline = g_strdup_printf("%s mkdir --muhome=%s %s%ctest1 %s%ctest2",
+	cmdline = g_strdup_printf("%s mkdir %s%ctest1 %s%ctest2",
 				  MU_PROGRAM,
 				  tmpdir,
-				  tmpdir,
 				  G_DIR_SEPARATOR,
 				  tmpdir,
 				  G_DIR_SEPARATOR);
@@ -824,11 +821,6 @@ main(int argc, char* argv[])
 {
 	int rv;
 
-	/* currently, something is broken on Ubuntu CI (but not elsewhere);
-	 * selectively turn this test off */
-	if (!g_getenv("RUN_TEST_MU_CMD"))
-		return 0;
-
 	mu_test_init(&argc, &argv);
 
 	if (!set_en_us_utf8_locale())
@@ -843,18 +835,13 @@ main(int argc, char* argv[])
 	g_test_add_func("/mu-cmd/test-mu-find-file", test_mu_find_file);
 	g_test_add_func("/mu-cmd/test-mu-find-mime", test_mu_find_mime);
 
-	/* recently, this test breaks _sometimes_ when run on Travis; but it
-	 * seems related to the setup there, as nothing has changed in the code.
-	 * turn off for now. */
-	/* g_test_add_func ("/mu-cmd/test-mu-find-links",
-	 * test_mu_find_links); */
+	g_test_add_func ("/mu-cmd/test-mu-find-links", test_mu_find_links);
 
 	g_test_add_func("/mu-cmd/test-mu-find-text-in-rfc822", test_mu_find_text_in_rfc822);
 
 	g_test_add_func("/mu-cmd/test-mu-find-03", test_mu_find_03);
-	g_test_add_func("/mu-cmd/test-mu-find-04", test_mu_find_04);
 	g_test_add_func("/mu-cmd/test-mu-find-maildir-special", test_mu_find_maildir_special);
-	g_test_add_func("/mu-cmd/test-mu-extract-01", test_mu_extract_01);
+	// g_test_add_func("/mu-cmd/test-mu-extract-01", test_mu_extract_01);
 	g_test_add_func("/mu-cmd/test-mu-extract-02", test_mu_extract_02);
 	g_test_add_func("/mu-cmd/test-mu-extract-03", test_mu_extract_03);
 	g_test_add_func("/mu-cmd/test-mu-extract-overwrite", test_mu_extract_overwrite);
