@@ -1,24 +1,24 @@
-;;; mu4e-lists.el -- part of mu4e, the mu mail user agent -*- lexical-binding: t -*-
+;;; mu4e-lists.el -- part of mu4e -*- lexical-binding: t -*-
 
-;; Copyright (C) 2011-2016 Dirk-Jan C. Binnema
+;; Copyright (C) 2011-2021 Dirk-Jan C. Binnema
 
 ;; Author: Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 ;; Maintainer: Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 
 ;; This file is not part of GNU Emacs.
 
-;; GNU Emacs is free software: you can redistribute it and/or modify
+;; mu4e is free software: you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
 ;; the Free Software Foundation, either version 3 of the License, or
 ;; (at your option) any later version.
 
-;; GNU Emacs is distributed in the hope that it will be useful,
+;; mu4e is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+;; along with mu4e.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -26,8 +26,9 @@
 ;; The shortname (friendly) should a at most 8 characters, camel-case
 
 ;;; Code:
-
-(defvar mu4e~mailing-lists
+
+;;; Configuration
+(defvar mu4e-mailing-lists
   '( ("bbdb-info.lists.sourceforge.net"                       . "BBDB")
      ("boost-announce.lists.boost.org"                        . "BoostA")
      ("boost-interest.lists.boost.org"                        . "BoostI")
@@ -81,22 +82,50 @@
      ("wl-en.ml.gentei.org"                                   . "WdrLust")
      ("xapian-devel.lists.xapian.org"                         . "Xapian")
      ("zsh-users.zsh.org"                                     . "ZshUsr"))
-  "AList of cells (MAILING-LIST-ID . SHORTNAME)")
+  "AList of cells (MAILING-LIST-ID . SHORTNAME).")
 
 (defcustom mu4e-user-mailing-lists nil
-  "An alist with cells (MAILING-LIST-ID . SHORTNAME); these are
-used in addition to the built-in list `mu4e~mailing-lists'."
+  "An alist with cells (MAILING-LIST-ID . SHORTNAME).
+These are used in addition to the built-in list `mu4e-mailing-lists'."
   :group 'mu4e-headers
   :type '(repeat (cons string string)))
 
-
 (defcustom mu4e-mailing-list-patterns nil
-  "A list of regex patterns to capture a shortname out of a list
-ID. For the first regex that matches, its first matchgroup will
-be used as the shortname."
+  "A list of regexps to capture a shortname out of a list-id.
+For the first regex that matches, its first matchgroup will be
+used as the shortname."
   :group 'mu4e-headers
   :type '(repeat (regexp)))
+
 
+(defvar mu4e--lists-hash nil
+  "Hashtable of mailing-list-id => shortname.
+Based on `mu4e-mailing-lists' and `mu4e-user-mailing-lists'.")
+
+
+(defun mu4e-get-mailing-list-shortname (list-id)
+  "Get the shortname for a mailing-list with list-id LIST-ID.
+Based on `mu4e-mailing-lists', `mu4e-user-mailing-lists', and
+`mu4e-mailing-list-patterns'."
+  (unless mu4e--lists-hash
+    (setq mu4e--lists-hash (make-hash-table :test 'equal))
+    (dolist (cell mu4e-mailing-lists)
+      (puthash (car cell) (cdr cell) mu4e--lists-hash))
+    (dolist (cell mu4e-user-mailing-lists)
+      (puthash (car cell) (cdr cell) mu4e--lists-hash)))
+  (or
+   (gethash list-id mu4e--lists-hash)
+   (and (boundp 'mu4e-mailing-list-patterns)
+        (seq-drop-while
+         (lambda (pattern)
+           (not (string-match pattern list-id)))
+         mu4e-mailing-list-patterns)
+        (match-string 1 list-id))
+   ;; if it's not in the db, take the part until the first dot if there is one;
+   ;; otherwise just return the whole thing
+   (if (string-match "\\([^.]*\\)\\." list-id)
+       (match-string 1 list-id)
+     list-id)))
 ;;; _
 (provide 'mu4e-lists)
 ;;; mu4e-lists.el ends here
