@@ -52,7 +52,7 @@ Example usage:
                     \"No date\")))
       (concat subject \" \" date)))
 
-  (setq org-mu4e-link-desc-func \='my-link-descr)"
+  (setq org-mu4e-link-desc-func \\='my-link-descr)"
   :type '(function)
   :group 'mu4e-org)
 
@@ -61,41 +61,49 @@ Example usage:
 If non-nil, `org-store-link' in `mu4e-headers-mode' links to the
 the current query; otherwise, it links to the message at point.")
 
+;; backward compat until org >= 9.3 is univeral.
+(defalias 'mu4e--org-link-store-props
+  (if (fboundp 'org-link-store-props)
+      #'org-link-store-props
+    (with-no-warnings
+      #'org-store-link-props)))
+
 (defun mu4e--org-store-link-query ()
   "Store a link to a mu4e query."
-  (setq org-store-link-plist nil) ; reset
-  (org-store-link-props
+  (setq org-store-link-plist nil)       ; reset
+  (mu4e--org-link-store-props
    :type        "mu4e"
    :query       (mu4e-last-query)
    :date        (format-time-string "%FT%T") ;; avoid error
    :link        (concat "mu4e:query:" (mu4e-last-query))
    :description (format "[%s]" (mu4e-last-query))))
 
-(defun mu4e--org-store-link-message ()
-  "Store a link to a mu4e message."
+(defun mu4e--org-store-link-message (&optional msg)
+  "Store a link to a mu4e message.
+If MSG is non-nil, store a link to MSG, otherwise use `mu4e-message-at-point'."
   (setq org-store-link-plist nil)
-  (let* ((msg      (mu4e-message-at-point))
+  (let* ((msg      (or msg (mu4e-message-at-point)))
          (from     (car-safe (plist-get msg :from)))
          (to       (car-safe (plist-get msg :to)))
          (date     (format-time-string "%FT%T" (plist-get msg :date)))
          (msgid    (or (plist-get msg :message-id)
                        (mu4e-error "Cannot link message without message-id")))
-	 (props `(:type  "mu4e"
-		  :date              ,date
-		  :from              ,(mu4e-contact-full from)
-		  :fromname          ,(mu4e-contact-name from)
-		  :fromnameoraddress ,(or (mu4e-contact-name from)
-				    	(mu4e-contact-email from)) ;; mu4e-specific
-		  :maildir           ,(plist-get msg :maildir)
-		  :message-id        ,msgid
-		  :path              ,(plist-get msg :path)
-		  :subject           ,(plist-get msg :subject)
-		  :to                ,(mu4e-contact-full to)
-		  :tonameoraddress   ,(or (mu4e-contact-name to)
-				    	(mu4e-contact-email to)) ;; mu4e-specific
-		  :link              ,(concat "mu4e:msgid:" msgid)
-		  :description       ,(funcall mu4e-org-link-desc-func msg))))
-    (apply #'org-store-link-props props)))
+         (props `(:type  "mu4e"
+                  :date              ,date
+                  :from              ,(mu4e-contact-full from)
+                  :fromname          ,(mu4e-contact-name from)
+                  :fromnameoraddress ,(or (mu4e-contact-name from)
+                                        (mu4e-contact-email from)) ;; mu4e-specific
+                  :maildir           ,(plist-get msg :maildir)
+                  :message-id        ,msgid
+                  :path              ,(plist-get msg :path)
+                  :subject           ,(plist-get msg :subject)
+                  :to                ,(mu4e-contact-full to)
+                  :tonameoraddress   ,(or (mu4e-contact-name to)
+                                        (mu4e-contact-email to)) ;; mu4e-specific
+                  :link              ,(concat "mu4e:msgid:" msgid)
+                  :description       ,(funcall mu4e-org-link-desc-func msg))))
+    (apply #'mu4e--org-link-store-props props)))
 
 (defun mu4e-org-store-link ()
   "Store a link to a mu4e message or query.
@@ -122,8 +130,6 @@ the query (for links starting with \"query:\")."
     (mu4e-search (match-string 1 link) current-prefix-arg))
    (t (mu4e-error "Unrecognized link type '%s'" link))))
 
-(make-obsolete 'org-mu4e-open 'mu4e-org-open "1.3.6")
-
 (defun mu4e-org-store-and-capture ()
   "Store a link to the current message or query.
 \(depending on `mu4e-org-link-query-in-headers-mode', and capture
@@ -132,9 +138,6 @@ it with org)."
   (call-interactively 'org-store-link)
   (org-capture))
 
-(make-obsolete 'org-mu4e-store-and-capture
-               'mu4e-org-store-and-capture "1.3.6")
-
 ;; install mu4e-link support.
 (org-link-set-parameters "mu4e"
                          :follow #'mu4e-org-open
